@@ -5,6 +5,9 @@ use clap::{ArgEnum, Args, Parser, Subcommand};
 use std::str::FromStr;
 use std::{error, fmt};
 
+mod interact;
+use interact::*;
+
 /// The current version of the CLI. (As defined in Cargo.toml)
 pub const SIGI_VERSION: &str = std::env!("CARGO_PKG_VERSION");
 
@@ -13,27 +16,58 @@ const DEFAULT_FORMAT: OutputFormat = OutputFormat::Human(NoiseLevel::Normal);
 const DEFAULT_BACKEND: Backend = Backend::HomeDir;
 const DEFAULT_SHORT_LIST_LIMIT: usize = 10;
 
+// === Glossary ===
+const COMPLETE_TERMS: [&str; 4] = ["complete", "done", "finish", "fulfill"];
+const COUNT_TERMS: [&str; 3] = ["count", "size", "length"];
+const DELETE_TERMS: [&str; 5] = ["delete", "pop", "remove", "cancel", "drop"];
+const DELETE_ALL_TERMS: [&str; 6] = [
+    "delete-all",
+    "purge",
+    "pop-all",
+    "remove-all",
+    "cancel-all",
+    "drop-all",
+];
+const HEAD_TERMS: [&str; 3] = ["head", "top", "first"];
+const IS_EMPTY_TERMS: [&str; 2] = ["is-empty", "empty"];
+const LIST_TERMS: [&str; 5] = ["list", "ls", "snoop", "show", "all"];
+const LIST_STACKS_TERMS: [&str; 2] = ["list-stacks", "stacks"];
+const MOVE_TERMS: [&str; 1] = ["move"];
+const MOVE_ALL_TERMS: [&str; 1] = ["move-all"];
+const NEXT_TERMS: [&str; 4] = ["next", "later", "cycle", "bury"];
+const PEEK_TERMS: [&str; 2] = ["peek", "show"];
+const PICK_TERMS: [&str; 1] = ["pick"];
+const PUSH_TERMS: [&str; 6] = ["push", "create", "add", "do", "start", "new"];
+const ROT_TERMS: [&str; 2] = ["rot", "rotate"];
+const SWAP_TERMS: [&str; 1] = ["swap"];
+const TAIL_TERMS: [&str; 3] = ["tail", "bottom", "last"];
+// === /glossary ===
+
 pub fn run() {
     let args = Cli::parse();
 
     let stack = args.stack.unwrap_or_else(|| DEFAULT_STACK_NAME.into());
 
-    if args.command.is_none() {
-        let output = args.fc.into_output_format().unwrap_or(DEFAULT_FORMAT);
-        let peek = StackEffect::Peek { stack };
-        peek.run(&DEFAULT_BACKEND, &output);
-        return;
-    }
-
-    let (effect, effect_fc) = args.command.unwrap().into_effect_and_fc(stack);
-
-    let output = args.fc.into_fallback_for(effect_fc);
-
-    effect.run(&DEFAULT_BACKEND, &output);
+    match args.mode {
+        None => {
+            let output = args.fc.into_output_format().unwrap_or(DEFAULT_FORMAT);
+            let peek = StackEffect::Peek { stack };
+            peek.run(&DEFAULT_BACKEND, &output);
+        }
+        Some(Mode::Command(command)) => {
+            let (effect, effect_fc) = command.into_effect_and_fc(stack);
+            let output = args.fc.into_fallback_for(effect_fc);
+            effect.run(&DEFAULT_BACKEND, &output);
+        }
+        Some(Mode::Interactive { fc }) => {
+            let output = args.fc.into_fallback_for(fc);
+            interact(stack, output);
+        }
+    };
 }
 
 #[derive(Parser)]
-#[clap(name = "sigi", version = SIGI_VERSION)]
+#[clap(name = "sigi", version = SIGI_VERSION, after_help = INTERACT_INSTRUCTIONS, after_long_help = INTERACT_LONG_INSTRUCTIONS)]
 /// An organizing tool for terminal lovers who hate organizing
 struct Cli {
     #[clap(flatten)]
@@ -44,41 +78,54 @@ struct Cli {
     stack: Option<String>,
 
     #[clap(subcommand)]
-    command: Option<Command>,
+    mode: Option<Mode>,
+}
+
+#[derive(Subcommand)]
+enum Mode {
+    /// Run in an interactive mode
+    #[clap(visible_alias = "i")]
+    Interactive {
+        #[clap(flatten)]
+        fc: FormatConfig,
+    },
+
+    #[clap(flatten)]
+    Command(Command),
 }
 
 #[derive(Subcommand)]
 enum Command {
     /// Move the current item to "<STACK>_history" and mark as completed
-    #[clap(visible_aliases = &["done", "finish", "fulfill"])]
+    #[clap(visible_aliases = &COMPLETE_TERMS[1..])]
     Complete {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// Print the total number of items in the stack
-    #[clap(visible_aliases = &["size", "length"])]
+    #[clap(visible_aliases = &COUNT_TERMS[1..])]
     Count {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// Move the current item to "<STACK>_history" and mark as deleted
-    #[clap(visible_aliases = &["pop", "remove", "cancel", "drop"])]
+    #[clap(visible_aliases = &DELETE_TERMS[1..])]
     Delete {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// Move all items to "<STACK>_history" and mark as deleted
-    #[clap(visible_aliases = &["purge", "pop-all", "remove-all", "cancel-all", "drop-all"])]
+    #[clap(visible_aliases = &DELETE_ALL_TERMS[1..])]
     DeleteAll {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// List the first N items (default is 10)
-    #[clap(visible_aliases = &["top", "first"])]
+    #[clap(visible_aliases = &HEAD_TERMS[1..])]
     Head {
         /// The number of items to display
         n: Option<usize>,
@@ -89,28 +136,28 @@ enum Command {
 
     /// Print "true" if stack has zero items, or print "false" (and exit with a
     /// nonzero exit code) if the stack does have items
-    #[clap(visible_aliases = &["empty"])]
+    #[clap(visible_aliases = &IS_EMPTY_TERMS[1..])]
     IsEmpty {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// List all items
-    #[clap(visible_aliases = &["ls", "snoop", "show", "all"])]
+    #[clap(visible_aliases = &LIST_TERMS[1..])]
     List {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// List all stacks
-    #[clap(visible_aliases = &["stacks"])]
+    #[clap(visible_aliases = &LIST_STACKS_TERMS[1..])]
     ListStacks {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// Move current item to another stack
-    #[clap(arg_required_else_help = true)]
+    #[clap(arg_required_else_help = true, visible_aliases = &MOVE_TERMS[1..])]
     Move {
         #[clap(name = "destination")]
         /// The stack that will get the source stack's current item
@@ -121,7 +168,7 @@ enum Command {
     },
 
     /// Move all items to another stack
-    #[clap(arg_required_else_help = true)]
+    #[clap(arg_required_else_help = true, visible_aliases = &MOVE_ALL_TERMS[1..])]
     MoveAll {
         #[clap(name = "destination")]
         /// The stack that will get all the source stack's items
@@ -132,20 +179,21 @@ enum Command {
     },
 
     /// Cycle to the next item; the current item becomes last
-    #[clap(visible_aliases = &["later", "cycle", "bury"])]
+    #[clap(visible_aliases = &NEXT_TERMS[1..])]
     Next {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// Show the first item. This is the default behavior when no command is given
-    #[clap(visible_aliases = &["show"])]
+    #[clap(visible_aliases = &PEEK_TERMS[1..])]
     Peek {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// Move items to the top of stack by their number
+    #[clap(visible_aliases = &PICK_TERMS[1..])]
     Pick {
         ns: Vec<usize>,
 
@@ -154,7 +202,7 @@ enum Command {
     },
 
     /// Create a new item
-    #[clap(visible_aliases = &["create", "add", "do", "start", "new"])]
+    #[clap(visible_aliases = &PUSH_TERMS[1..])]
     Push {
         // The content to add as an item. Multiple arguments will be interpreted as a single string
         content: Vec<String>,
@@ -164,20 +212,21 @@ enum Command {
     },
 
     /// Rotate the three most-current items
-    #[clap(visible_aliases = &["rotate"])]
+    #[clap(visible_aliases = &ROT_TERMS[1..])]
     Rot {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// Swap the two most-current items
+    #[clap(visible_aliases = &SWAP_TERMS[1..])]
     Swap {
         #[clap(flatten)]
         fc: FormatConfig,
     },
 
     /// List the last N items (default is 10)
-    #[clap(visible_aliases = &["bottom", "last"])]
+    #[clap(visible_aliases = &TAIL_TERMS[1..])]
     Tail {
         /// The number of items to display
         n: Option<usize>,
@@ -189,32 +238,33 @@ enum Command {
 
 impl Command {
     fn into_effect_and_fc(self, stack: String) -> (StackEffect, FormatConfig) {
+        use StackEffect::*;
         match self {
-            Command::Complete { fc } => (StackEffect::Complete { stack }, fc),
-            Command::Count { fc } => (StackEffect::Count { stack }, fc),
-            Command::Delete { fc } => (StackEffect::Delete { stack }, fc),
-            Command::DeleteAll { fc } => (StackEffect::DeleteAll { stack }, fc),
+            Command::Complete { fc } => (Complete { stack }, fc),
+            Command::Count { fc } => (Count { stack }, fc),
+            Command::Delete { fc } => (Delete { stack }, fc),
+            Command::DeleteAll { fc } => (DeleteAll { stack }, fc),
             Command::Head { n, fc } => {
                 let n = n.unwrap_or(DEFAULT_SHORT_LIST_LIMIT);
-                (StackEffect::Head { n, stack }, fc)
+                (Head { n, stack }, fc)
             }
-            Command::IsEmpty { fc } => (StackEffect::IsEmpty { stack }, fc),
-            Command::List { fc } => (StackEffect::ListAll { stack }, fc),
-            Command::ListStacks { fc } => (StackEffect::ListStacks, fc),
-            Command::Move { dest, fc } => (StackEffect::Move { stack, dest }, fc),
-            Command::MoveAll { dest, fc } => (StackEffect::MoveAll { stack, dest }, fc),
-            Command::Next { fc } => (StackEffect::Next { stack }, fc),
-            Command::Peek { fc } => (StackEffect::Peek { stack }, fc),
-            Command::Pick { ns, fc } => (StackEffect::Pick { stack, indices: ns }, fc),
+            Command::IsEmpty { fc } => (IsEmpty { stack }, fc),
+            Command::List { fc } => (ListAll { stack }, fc),
+            Command::ListStacks { fc } => (ListStacks, fc),
+            Command::Move { dest, fc } => (Move { stack, dest }, fc),
+            Command::MoveAll { dest, fc } => (MoveAll { stack, dest }, fc),
+            Command::Next { fc } => (Next { stack }, fc),
+            Command::Peek { fc } => (Peek { stack }, fc),
+            Command::Pick { ns, fc } => (Pick { stack, indices: ns }, fc),
             Command::Push { content, fc } => {
                 let content = content.join(" ");
-                (StackEffect::Push { stack, content }, fc)
+                (Push { stack, content }, fc)
             }
-            Command::Rot { fc } => (StackEffect::Rot { stack }, fc),
-            Command::Swap { fc } => (StackEffect::Swap { stack }, fc),
+            Command::Rot { fc } => (Rot { stack }, fc),
+            Command::Swap { fc } => (Swap { stack }, fc),
             Command::Tail { n, fc } => {
                 let n = n.unwrap_or(DEFAULT_SHORT_LIST_LIMIT);
-                (StackEffect::Tail { n, stack }, fc)
+                (Tail { n, stack }, fc)
             }
         }
     }
@@ -247,20 +297,24 @@ impl FormatConfig {
             quiet,
             format,
         } = self;
+
+        use NoiseLevel::*;
+        use OutputFormat::*;
+
         format
             .map(|format| match format {
-                ProgrammaticFormat::Csv => OutputFormat::Csv,
-                ProgrammaticFormat::Json => OutputFormat::Json,
-                ProgrammaticFormat::JsonCompact => OutputFormat::JsonCompact,
-                ProgrammaticFormat::Tsv => OutputFormat::Tsv,
+                ProgrammaticFormat::Csv => Csv,
+                ProgrammaticFormat::Json => Json,
+                ProgrammaticFormat::JsonCompact => JsonCompact,
+                ProgrammaticFormat::Tsv => Tsv,
             })
             .or_else(|| {
                 if verbose {
-                    Some(OutputFormat::Human(NoiseLevel::Verbose))
+                    Some(Human(Verbose))
                 } else if silent {
-                    Some(OutputFormat::Silent)
+                    Some(Silent)
                 } else if quiet {
-                    Some(OutputFormat::Human(NoiseLevel::Quiet))
+                    Some(Human(Quiet))
                 } else {
                     None
                 }
@@ -285,13 +339,17 @@ enum ProgrammaticFormat {
 
 impl FromStr for ProgrammaticFormat {
     type Err = UnknownFormat;
+
     fn from_str(format: &str) -> Result<Self, Self::Err> {
+        use ProgrammaticFormat::*;
+
         let format = format.to_ascii_lowercase();
+
         match format.as_str() {
-            "csv" => Ok(Self::Csv),
-            "json" => Ok(Self::Json),
-            "json-compact" => Ok(Self::JsonCompact),
-            "tsv" => Ok(Self::Tsv),
+            "csv" => Ok(Csv),
+            "json" => Ok(Json),
+            "json-compact" => Ok(JsonCompact),
+            "tsv" => Ok(Tsv),
             _ => Err(UnknownFormat { format }),
         }
     }
