@@ -32,6 +32,14 @@ impl OutputFormat {
         dt.to_rfc2822()
     }
 
+    pub fn is_nonquiet_for_humans(&self) -> bool {
+        match self {
+            OutputFormat::Human(NoiseLevel::Quiet) => false,
+            OutputFormat::Human(_) => true,
+            _ => false,
+        }
+    }
+
     pub fn log(&self, labels: Vec<&str>, values: Vec<Vec<&str>>) {
         if let OutputFormat::Silent = self {
             return;
